import datetime

inputs = {
    "r0":                       {"time": "daily", "array_name": "r0"},
    "ndvi":                     {"time": "daily", "array_name": "ndvi"}, 
    "lst":                      {"time": "daily", "array_name": "lst"},
    "dtime":                     {"time": "daily", "array_name": "dtime"},
    "lat_deg":                      {"time": "static", "array_name": "lat_deg"},
    "lon_deg":                      {"time": "static", "array_name": "lon_deg"},
    "z":                      {"time": "static", "array_name": "z"},
    "slope_deg":                    {"time": "static", "array_name": "slope_deg"},
    "aspect_deg":                   {"time": "static", "array_name": "aspect_deg"},
    "land_mask":                 {"time": "yearly", "array_name": "land_mask"},
    "lue_max":                   {"time": "yearly", "array_name": "lue_max"},
    "p_air_24_0":                {"time": "daily", "array_name": "p_air_0_24"},
    "Pair_inst_0":              {"time": "daily", "array_name": "p_air_0_i"},
    "Pair_inst":                {"time": "daily", "array_name": "p_air_i"},
    "P_24":            {"time": "daily", "array_name": "P_24"},
    "qv_24":                    {"time": "daily", "array_name": "qv_24"},
    "qv_inst":                  {"time": "daily", "array_name": "qv_i"},
    "t_air_24":                  {"time": "daily", "array_name": "t_air_24"},
    "t_air_max_24":              {"time": "daily", "array_name": "t_air_max_24"},
    "t_air_min_24":              {"time": "daily", "array_name": "t_air_min_24"},
    "tair_inst":                {"time": "daily", "array_name": "t_air_i"},
    "t_amp_year":                 {"time": "yearly", "array_name": "t_amp_year"},
    "u_24":                  {"time": "daily", "array_name": "u_24"},
    "wind_inst":                {"time": "daily", "array_name": "u_i"},
    "wv_inst":                  {"time": "daily", "array_name": "wv_i"},
    "trans_24":                 {"time": "daily", "array_name": "trans_24"},
    "se_root":                  {"time": "daily", "array_name": "se_root"},
    "z_obst_max":  {"time": "yearly", "array_name": "z_obst_max"},
    "rs_min": {"time": "yearly", "array_name": "rs_min"},
    "SR_24":                    {"time": "daily", "array_name": "ra_24"},
    "z_oro":                    {"time": "daily", "array_name": "z_oro"},
    "lw_offset":                {"time": "static", "array_name": "lw_offset"},
    "lw_slope":                 {"time": "static", "array_name": "lw_slope"},
    "r0_bare":                  {"time": "static", "array_name": "r0_bare"},
    "r0_full":                  {"time": "static", "array_name": "r0_full"},
    "rn_offset":                {"time": "static", "array_name": "rn_offset"},
    "rn_slope":                 {"time": "static", "array_name": "rn_slope"},
    "t_opt":                    {"time": "static", "array_name": "t_opt"},
    "vpd_slope":                {"time": "static", "array_name": "vpd_slope"},
    "u2m_24":                      {"time": "daily", "array_name": "u2m_24"},
    "v2m_24":                      {"time": "daily", "array_name": "v2m_24"},
    "ra_24":                    {"time": "daily", "array_name": "ra_24"},
}

outputs = {
    'B0c':                  {'file_name': 'B0c'},
    'Bhc':                  {'file_name': 'Bhc'},
    'Dhc':                  {'file_name': 'Dhc'},
    'Tl2':                  {'file_name': 'Tl2'},
    'ad_24':                {'file_name': 'ad_24'},
    'ad_dry_24':            {'file_name': 'ad_dry_24'},
    'ad_dry_i':             {'file_name': 'ad_dry_i'},
    'ad_i':                 {'file_name': 'ad_i'},
    'ad_moist_24':          {'file_name': 'ad_moist_24'},
    'ad_moist_i':           {'file_name': 'ad_moist_i'},
    'aspect':               {'file_name': 'aspect'},
    'biomass_prod':         {'file_name': 'biomass_prod_kg-ha'}, #
    'diffusion_index':      {'file_name': 'diffusion_index'},
    'disp':                 {'file_name': 'disp'},
    'e_24':                 {'file_name': 'e_24'},
    'e_24_init':            {'file_name': 'e_24_init'},
    'e_24_mm':              {'file_name': 'e_24_mm'},
    'emiss_atm_i':          {'file_name': 'emiss_atm_i'},
    'et_24_mm':             {'file_name': 'et_24_mm'},
    'et_ref_24':            {'file_name': 'et_ref_24'},
    'et_ref_24_mm':         {'file_name': 'et_ref_24_mm'},
    'g0_24':                {'file_name': 'g0_24'},
    'g0_bs':                {'file_name': 'g0_bs'},
    'h0':                   {'file_name': 'h0'},
    'h0ref':                {'file_name': 'h0ref'},
    'h_canopy_24_init':     {'file_name': 'h_canopy_24_init'},
    'h_soil_24_init':       {'file_name': 'h_soil_24_init'},
    'ha':                   {'file_name': 'ha'},
    'int_mm':               {'file_name': 'int_mm'},
    'int_wm2':              {'file_name': 'int_wm2'},
    'l_net':                {'file_name': 'l_net'},
    'lai':                  {'file_name': 'LAI'}, #
    'lai_eff':              {'file_name': 'LAI_eff'}, #
    'lat':                  {'file_name': 'lat'},
    'lh_24':                {'file_name': 'lh_24'},
    'lon':                  {'file_name': 'lon'},
    'lst_max':              {'file_name': 'lst_max'},
    'lst_min':              {'file_name': 'lst_min'},
    'm':                    {'file_name': 'm'},
    'p_air_24':             {'file_name': 'p_air_24'},
    'psy_24':               {'file_name': 'psy_24'},
    'r_canopy':             {'file_name': 'r_canopy'},
    'r_canopy_0':           {'file_name': 'r_canopy_0'},
    'r_soil':               {'file_name': 'r_soil'},
    'ra_24':                {'file_name': 'ra_24'},
    'ra_24_toa':            {'file_name': 'ra_24_toa'},
    'ra_24_toa_flat':       {'file_name': 'ra_24_toa_flat'}, #
    'ra_canopy_init':       {'file_name': 'ra_canopy_init'},
    'ra_hor_clear_i':       {'file_name': 'ra_hor_clear_i'},
    'ra_soil_init':         {'file_name': 'ra_soil_init'},
    'raa':                  {'file_name': 'raa'},
    'rac':                  {'file_name': 'rac'},
    'ras':                  {'file_name': 'ras'},
    'rn_24':                {'file_name': 'rn_24'},
    'rn_24_canopy':         {'file_name': 'rn_24_canopy'},
    'rn_24_grass':          {'file_name': 'rn_24_grass'},
    'rn_24_soil':           {'file_name': 'rn_24_soil'},
    'rn_bare':              {'file_name': 'rn_bare'},
    'rn_full':              {'file_name': 'rn_full'},
    'rotm':                 {'file_name': 'rotm'},
    'se_root':              {'file_name': 'se_root'},
    'sf_soil':              {'file_name': 'sf_soil'},
    'slope':                {'file_name': 'slope'},
    'ssvp_24':              {'file_name': 'ssvp_24'},
    'stress_moist':         {'file_name': 'stress_moist'},
    'stress_rad':           {'file_name': 'stress_rad'},
    'stress_temp':          {'file_name': 'stress_temp'},
    'stress_vpd':           {'file_name': 'stress_vpd'},
    'svp_24':               {'file_name': 'svp_24'},
    't_24':                 {'file_name': 't_24'},
    't_24_init':            {'file_name': 't_24_init'},
    't_24_mm':              {'file_name': 't_24_mm'},
    'tpot_24':              {'file_name': 'tpot_24'},
    'tpot_24_mm':           {'file_name': 'tpot_24_mm'},
    't_air_k_24':           {'file_name': 't_air_k_24'},
    't_air_k_i':            {'file_name': 't_air_k_i'},
    't_dew_i':              {'file_name': 't_dew_i'},
    't_max_bare':           {'file_name': 't_max_bare'},
    't_max_full':           {'file_name': 't_max_full'},
    't_min_bare':           {'file_name': 't_min_bare'},
    't_min_full':           {'file_name': 't_min_full'},
    't_wet_i':              {'file_name': 't_wet_i'},
    't_wet_k_i':            {'file_name': 't_wet_k_i'},
    'u_b_24':               {'file_name': 'u_b_24'},
    'u_b_i_bare':           {'file_name': 'u_b_i_bare'},
    'u_b_i_full':           {'file_name': 'u_b_i_full'},
    'u_i_soil':             {'file_name': 'u_i_soil'},
    'u_star_24_init':       {'file_name': 'u_star_24_init'},
    'u_star_24_soil_init':  {'file_name': 'u_star_24_soil_init'},
    'u_star_i_bare':        {'file_name': 'u_star_i_bare'},
    'u_star_i_full':        {'file_name': 'u_star_i_full'},
    'vc':                   {'file_name': 'vc'},
    'vp_24':                {'file_name': 'vp_24'},
    'vp_i':                 {'file_name': 'vp_i'},
    'vpd_24':               {'file_name': 'vpd_24'},
    'w_i':                  {'file_name': 'w_i'},
    'ws':                   {'file_name': 'ws'},
    'z0m':                  {'file_name': 'z0m'},
    'z_obst':               {'file_name': 'z_obst'},
    'z_oro':                {'file_name': 'z_oro'},
    'L_bare':               {'file_name': 'L_bare'},
    'L_full':               {'file_name': 'L_full'},
    'trans_24':             {'file_name': 'trans_24'}
}

order = [
    "vc",
    "lai",
    "lai_eff",
    "sf_soil",
    "lat",
    "ws",
    "slope",
    "aspect",
    "ra_24_toa",
    "ra_24_toa_flat",
    "diffusion_index",
    "ra_24",
    "trans_24",#
    "stress_rad",
    "p_air_0_24",#
    "p_air_24",
    "vp_24",
    "t_air_24",
    "svp_24",
    "vpd_24",
    "stress_vpd",
    "stress_temp",
    "rs_min",#
    "r_canopy_0",
    "t_air_k_24",
    "l_net",
    "int_mm",
    "lh_24",
    "int_wm2",
    "rn_24",
    "rn_24_canopy",

    "t_air_k_i",
    "vp_i",
    "ad_moist_i",
    "ad_dry_i",
    "ad_i",
    "u_b_i_bare",
    "lon",
    "ha",
    "h0",
    "h0ref",
    "m",
    "rotm",
    "Tl2",
    "B0c",
    "Bhc",
    "Dhc",
    "ra_hor_clear_i",
    "emiss_atm_i",
    "rn_bare",
    "rn_full",
    "h_bare",
    "h_full",
    "u_b_i_full",
    "u_star_i_bare",
    "u_star_i_full",
    "L_bare",
    "L_full",
    "u_i_soil",
    "ras",
    "raa",
    "rac",
    "t_max_bare",
    "t_max_full",
    "w_i",
    "t_dew_i",
    "t_wet_i",
    "lst_max",
    "t_wet_k_i",
    "lst_min",
    "t_min_bare",
    "t_min_full",
    "lst_min",

    "se_root",
    "stress_moist",
    "r_canopy",
    "z_oro",
    "z_obst",
    "z0m",
    "ra_canopy_init",
    "u_b_24",
    "ad_dry_24",
    "ad_moist_24",
    "ad_24",
    "psy_24",
    "ssvp_24",
    "t_24_init",
    "h_canopy_24_init",
    "disp",
    "u_star_24_init",
    "t_24",
    "t_24_mm",
    "tpot_24",
    "tpot_24_mm",
    "rn_24_soil",
    "r_soil",
    "ra_soil_init",
    "u_star_24_soil_init",
    "g0_bs",
    "g0_24",
    "e_24_init",
    "h_soil_24_init",
    "e_24",
    "e_24_mm",
    "et_24_mm",
    "rn_24_grass",
    "et_ref_24",
    "et_ref_24_mm",
    "lue",
    "fpar",
    "apar",
    "biomass_prod_kg-ha",
]

def get_raw_meteo_paths(type = "all"):

    meteo_raw_paths_GEOS = {
        "Pair_24_0":    ("{raw_folder}", "GEOS5", "Sea_Level_Pressure", "daily", "slp_GEOS_kpa_daily_{date}.tif"),
        "Pair_inst_0":  ("{raw_folder}", "GEOS5", "Sea_Level_Pressure", "three_hourly", "slp_GEOS_kpa_3-hourly_{date}_H{hour}.M00.tif"),
        "Pair_inst":    ("{raw_folder}", "GEOS5", "Surface_Pressure", "three_hourly", "ps_GEOS_kpa_3-hourly_{date}_H{hour}.M00.tif"),
        "qv_24":        ("{raw_folder}", "GEOS5", "Specific_Humidity", "daily", "qv2m_GEOS_kg-kg-1_daily_{date}.tif"),
        "qv_inst":      ("{raw_folder}", "GEOS5", "Specific_Humidity", "three_hourly", "qv2m_GEOS_kg-kg-1_3-hourly_{date}_H{hour}.M00.tif"),
        "tair_24":      ("{raw_folder}", "GEOS5", "Air_Temperature", "daily", "t2m_GEOS_K_daily_{date}.tif"),
        "tair_inst":    ("{raw_folder}", "GEOS5", "Air_Temperature", "three_hourly", "t2m_GEOS_K_3-hourly_{date}_H{hour}.M00.tif"),
        "tair_max_24":  ("{raw_folder}", "GEOS5", "Air_Temperature", "daily", "t2m_GEOS_K_daily-max_{date}.tif"),
        "tair_min_24":  ("{raw_folder}", "GEOS5", "Air_Temperature", "daily", "t2m_GEOS_K_daily-min_{date}.tif"),
        "wv_inst":      ("{raw_folder}", "GEOS5", "Total_Precipitable_Water_Vapor", "three_hourly", "tqv_GEOS_mm_3-hourly_{date}_H{hour}.M00.tif"),
        "v2m_inst":     ("{raw_folder}", "GEOS5", "Northward_Wind", "three_hourly", "v2m_GEOS_m-s-1_3-hourly_{date}_H{hour}.M00.tif"),
        "u2m_inst":     ("{raw_folder}", "GEOS5", "Eastward_Wind", "three_hourly", "u2m_GEOS_m-s-1_3-hourly_{date}_H{hour}.M00.tif"),
        "v2m_24":       ("{raw_folder}", "GEOS5", "Northward_Wind", "daily", "v2m_GEOS_m-s-1_daily_{date}.tif"),
        "u2m_24":       ("{raw_folder}", "GEOS5", "Eastward_Wind", "daily", "u2m_GEOS_m-s-1_daily_{date}.tif"),
    }

    meteo_raw_paths_MERRA = {
        "Pair_24_0":    ("{raw_folder}", "MERRA2", "Sea_Level_Pressure", "daily_MERRA2", "slp_MERRA_kpa_daily_{date}.tif"),
        "Pair_inst_0":  ("{raw_folder}", "MERRA2", "Sea_Level_Pressure", "hourly_MERRA2", "slp_MERRA_kpa_hourly_{date}_H{hour}.M00.tif"),
        "Pair_inst":    ("{raw_folder}", "MERRA2", "Surface_Pressure", "hourly_MERRA2", "ps_MERRA_kpa_hourly_{date}_H{hour}.M00.tif"),
        "qv_24":        ("{raw_folder}", "MERRA2", "Specific_Humidity", "daily_MERRA2", "q2m_MERRA_kg-kg-1_daily_{date}.tif"),
        "qv_inst":      ("{raw_folder}", "MERRA2", "Specific_Humidity", "hourly_MERRA2", "q2m_MERRA_kg-kg-1_hourly_{date}_H{hour}.M00.tif"),
        "tair_24":      ("{raw_folder}", "MERRA2", "Air_Temperature", "daily_MERRA2", "t2m_MERRA_K_daily_{date}.tif"),
        "tair_inst":    ("{raw_folder}", "MERRA2", "Air_Temperature", "hourly_MERRA2", "t2m_MERRA_K_hourly_{date}_H{hour}.M00.tif"),
        "tair_max_24":  ("{raw_folder}", "MERRA2", "Air_Temperature", "daily_MERRA2", "t2mmax_MERRA_K_daily_{date}.tif"),
        "tair_min_24":  ("{raw_folder}", "MERRA2", "Air_Temperature", "daily_MERRA2", "t2mmin_MERRA_K_daily_{date}.tif"),
        "wv_inst":      ("{raw_folder}", "MERRA2", "Total_Precipitable_Water_Vapor", "hourly_MERRA2", "tpw_MERRA_mm_hourly_{date}_H{hour}.M00.tif"),
        "v2m_inst":     ("{raw_folder}", "MERRA2", "Northward_Wind", "hourly_MERRA2", "v2m_MERRA_m-s-1_hourly_{date}_H{hour}.M00.tif"),
        "u2m_inst":     ("{raw_folder}", "MERRA2", "Eastward_Wind", "hourly_MERRA2", "u2m_MERRA_m-s-1_hourly_{date}_H{hour}.M00.tif"),
        "v2m_24":       ("{raw_folder}", "MERRA2", "Northward_Wind", "daily_MERRA2", "v2m_MERRA_m-s-1_daily_{date}.tif"),
        "u2m_24":       ("{raw_folder}", "MERRA2", "Eastward_Wind", "daily_MERRA2", "u2m_MERRA_m-s-1_daily_{date}.tif"),
        "u2m_24":       ("{raw_folder}", "MERRA2", "Eastward_Wind", "daily_MERRA2", "u2m_MERRA_m-s-1_daily_{date}.tif"),
    }

    if type == "all":
        raw_meteo_paths = {"MERRA2": meteo_raw_paths_MERRA,
                        "GEOS5": meteo_raw_paths_GEOS}
    if type == "inst":
        raw_meteo_paths = {"MERRA2": {k: v for k,v in meteo_raw_paths_MERRA.items() if "inst" in k},
                        "GEOS5": {k: v for k,v in meteo_raw_paths_GEOS.items() if "inst" in k}}   

    if type == "non-inst":
        raw_meteo_paths = {"MERRA2": {k: v for k,v in meteo_raw_paths_MERRA.items() if "inst" not in k},
                "GEOS5": {k: v for k,v in meteo_raw_paths_GEOS.items() if "inst" not in k}}   

    return raw_meteo_paths

def get_source_validations():
    
    valid_sources = {
        "METEO": ["GEOS5", "MERRA2"],
        "NDVI": ["MOD13", "MYD13", "PROBAV"],
        "ALBEDO": ["MDC43", "PROBAV"],
        "LST": ["MOD11", "MYD11"],
        "LULC": ["GLOBCOVER", "WAPOR"],
        "DEM": ["SRTM",],
        "PRECIPITATION": ["CHIRPS",],
        "TRANS": ["MERRA2"],
    }

    valid_dates = {
        "GEOS5": (datetime.date(2017,12,1), datetime.date.today()),
        "MERRA2": (datetime.date(1980,1,1), datetime.date.today()),
        "MOD13": (datetime.date(2000,2,18), datetime.date.today()),
        "MYD13": (datetime.date(2002,7,4), datetime.date.today()),
        "PROBAV": (datetime.date(2014,3,11), datetime.date.today()),
        "MDC43": (datetime.date(2000,2,16), datetime.date.today()),
        "MOD11": (datetime.date(2000,2,24), datetime.date.today()),
        "MYD11": (datetime.date(2002,7,4), datetime.date.today()),
        "GLOBCOVER": (datetime.date(2009,1,1), datetime.date(2009,1,1)),
        "WAPOR": (datetime.date(2009,1,1), datetime.date(2020,1,1)),
        "CHIRPS": (datetime.date(1981,1,1), datetime.date.today()),
        "SRTM": (datetime.date(2009,1,1), datetime.date(2009,1,1)),
    }

    return valid_sources, valid_dates

def get_source_level_selections():

    source_selection_level2 = {
        "METEO": ["GEOS5"],
        "NDVI": ["PROBAV"],
        "ALBEDO": ["PROBAV"],
        "LST": ["MOD11", "MYD11"],
        "LULC": ["WAPOR"],
        "DEM": ["SRTM"],
        "PRECIPITATION": ["CHIRPS"],
        "TRANS": ["MERRA2"],
    }

    source_selection_level1 = {
        "METEO": ["GEOS5"],
        "NDVI": ["MOD13", "MYD13"],
        "ALBEDO": ["MDC43"],
        "LST": ["MOD11", "MYD11"],
        "LULC": ["WAPOR"],
        "DEM": ["SRTM"],
        "PRECIPITATION": ["CHIRPS"],
        "TRANS": ["MERRA2"],
    }

    levels = {"level_1": source_selection_level1, 
              "level_2": source_selection_level2}

    return levels

def get_password_reqs():
    password_reqs = {"NASA": ["MOD13", "MYD13", "MCD43", 
                                "MOD11", "MYD11", "MERRA2"],
                    "VITO": ["PROBAV"],
                    "WAPOR": ["WAPOR"],}
    return password_reqs

def get_temp_input_data_reqs():
    temporal_input_data_req = [
        {"name": "Albedo",
        "unit": "[-]",
        "quantity": "Albedo",
        "level": "surface",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/ALBEDO_{year}{month}{day}.tif"},

        {"name": "Land Surface Temperature",
        "unit": "[K]",
        "quantity": "Temperature",
        "level": "surface",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/LST_{year}{month}{day}.tif"},

        {"name": "Normalized Difference Vegetation Index",
        "unit": "[-]",
        "quantity": "Normalized Difference Vegetation Index",
        "level": "surface",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/NDVI_{year}{month}{day}.tif"},

        {"name": "Air Pressure at sea level (daily average)",
        "unit": "[kPa]",
        "quantity": "Air Pressure",
        "level": "sea",
        "time": "daily average",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/Pair_24_0_{year}{month}{day}.tif"},

        {"name": "Air Pressure at sea level (instanteneous)", 
        "unit": "[kPa]",
        "quantity": "Air Pressure",
        "level": "sea",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/Pair_inst_0_{year}{month}{day}.tif"},

        {"name": "Air Pressure at surface level (instanteneous)",
        "unit": "[kPa]",
        "quantity": "Air Pressure",
        "level": "surface",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/Pair_inst_{year}{month}{day}.tif"},

        {"name": "Precipitation",
        "unit": "[mm/day]",
        "quantity": "Precipitation",
        "level": "surface",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/Precipitation_{year}{month}{day}.tif"},

        {"name": "Specific Humidity (daily average)", 
        "unit": "[kg/kg]",
        "quantity": "Specific Humidity",
        "level": "surface",
        "time": "daily average",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/qv_24_{year}{month}{day}.tif"},

        {"name": "Specific Humidity (instanteneous)", "unit": "[kg/kg]",
        "quantity": "Specific Humidity",
        "level": "surface",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/qv_inst_{year}{month}{day}.tif"},

        {"name": "Air Temperature (daily average)", "unit": "[C]",
        "quantity": "Temperature",
        "level": "surface",
        "time": "daily average",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/tair_24_{year}{month}{day}.tif"},

        {"name": "Air Temperature (instanteneous)", "unit": "[C]",
        "quantity": "Temperature",
        "level": "surface",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/tair_inst_{year}{month}{day}.tif"},

        {"name": "Air Temperature (daily maximum)", "unit": "[C]",
        "quantity": "Temperature",
        "level": "surface",
        "time": "daily maximum",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/tair_max_24_{year}{month}{day}.tif"},

        {"name": "Air Temperature (daily minimum)", "unit": "[C]",
        "quantity": "Temperature",
        "level": "surface",
        "time": "daily minimum",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/tair_min_24_{year}{month}{day}.tif"},

        {"name": "Transmissivity", "unit": "[-]",
        "quantity": "Transmissivity",
        "level": "column",
        "time": "daily average",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/Trans_24_{year}{month}{day}.tif"},

        {"name": "Windspeed (daily average)", "unit": "[m/s]",
        "quantity": "Windspeed",
        "level": "surface",
        "time": "daily average",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/wind_24_{year}{month}{day}.tif"},

        {"name": "Windspeed (instanteneous)", "unit": "[m/s]",
        "quantity": "Windspeed",
        "level": "surface",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/wind_inst_{year}{month}{day}.tif"},

        {"name": "Total Precipitable Water Vapour", "unit": "[mm]",
        "quantity": "Total Precipitable Water Vapour",
        "level": "column",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/wv_inst_{year}{month}{day}.tif"},

        {"name": "Instantaneous Data Time", "unit": "[hour]",
        "quantity": "Time",
        "level": "surface",
        "time": "instanteneous",
        "filepath": "{input_folder}/{level}/{year}{month}{day}/Time_{year}{month}{day}.tif"},
    ]

    return temporal_input_data_req

def get_static_input_data_reqs():
    static_input_data_req = [
        {"name": "Latitude",
        "unit": "[DD]",
        "quantity": "Latitude",
        "level": "surface",
        "time": "invariant",
        "filepath": "{input_folder}/{level}/static/Lat.tif"},

        {"name": "Longitude",
        "unit": "[DD]",
        "quantity": "Longitude",
        "level": "surface",
        "time": "invariant",
        "filepath": "{input_folder}/{level}/static/Lon.tif"},

        {"name": "Slope",
        "unit": "[degrees]",
        "quantity": "Slope",
        "level": "surface",
        "time": "invariant",
        "filepath": "{input_folder}/{level}/static/Slope.tif"},

        {"name": "Slope Aspect",
        "unit": "[degrees]",
        "quantity": "Aspect",
        "level": "surface",
        "time": "invariant",
        "filepath": "{input_folder}/{level}/static/Aspect.tif"},

        {"name": "Bulk Stomatal Resistance",
        "unit": "[s/m]",
        "quantity": "Resistance",
        "level": "surface",
        "time": "yearly",
        "filepath": "{input_folder}/{level}/static/Bulk_Stomatal_resistance_{year}.tif"},

        {"name": "Digital Elevation Model",
        "unit": "[m.a.s.l]",
        "quantity": "Altitude",
        "level": "surface",
        "time": "invariant",
        "filepath": "{input_folder}/{level}/static/DEM.tif"},

        {"name": "Landmask",
        "unit": "[-]",
        "quantity": "Landmask",
        "level": "surface",
        "time": "yearly",
        "filepath": "{input_folder}/{level}/static/LandMask_{year}.tif"},

        {"name": "Maximum Light Use Efficiency",
        "unit": "[gr/MJ]",
        "quantity": "Efficiency",
        "level": "surface",
        "time": "yearly",
        "filepath": "{input_folder}/{level}/static/LUEmax_{year}.tif"},

        {"name": "Maximum Obstacle Height",
        "unit": "[m]",
        "quantity": "Height",
        "level": "surface",
        "time": "yearly",
        "filepath": "{input_folder}/{level}/static/Maximum_Obstacle_Height_{year}.tif"},

        {"name": "Air Temperature (yearly amplitude)",
        "unit": "[K]",
        "quantity": "Temperature",
        "level": "surface",
        "time": "yearly",
        "filepath": "{input_folder}/{level}/static/Tair_amp_{year}.tif"},

    ]
    return static_input_data_req


# -*- coding: utf-8 -*-
"""
Swets NDVI filtering
author: Laust Færch @ DHI GRAS
Created on 2020/08/29

Based on the article:
Swets, D.L, Reed, B.C., Rowland, J.D., Marko, S.E., 1999. A weighted least-squares approach to temporal
NDVI smoothing. In: Proceedings of the 1999 ASPRS Annual Conference, Portland, Oregon, pp. 526-536.

"""

import numpy as np
from numba import jit
from scipy import interpolate
from scipy.ndimage.filters import generic_filter


def _interpolate1d(data):

    # we need at least 2 non-nan elements
    if np.sum(~np.isnan(data)*1) < 2:
        return data

    good = ~np.isnan(data)

    # scipy interpolation
    finterp = interpolate.interp1d(np.flatnonzero(good),
                                   data[good], kind='linear',
                                   fill_value=np.nan,
                                   bounds_error=False)

    yinterp = finterp(np.arange(data.shape[0]))

    return yinterp


# calculate the weight of each sample based on neighbours
def _calc_weights(y):
    # for class local_peak, sloping_points, local_valley
    class_weights = [1.5, 0.5, 0.005] # weights defined in article

    left_shift = (y - np.roll(y, -1)) >= 0
    right_shift = (y - np.roll(y, 1)) >= 0

    peaks = left_shift & right_shift
    valleys = (~left_shift) & (~right_shift)
    slopes = (~peaks) & (~valleys)

    weights = np.zeros_like(y)

    weights[peaks] = class_weights[0]
    weights[slopes] = class_weights[1]
    weights[valleys] = class_weights[2]

    return weights


# calculate the weighted linear regression
@jit(nopython=True)
def _calc_linreg(x, y, w):
    eps = 1e-8

    sw = np.sum(w)
    sy = np.sum(w * y)
    sx = np.sum(w * x)
    sxy = np.sum(w * x * y)
    sx2 = np.sum(w * x ** 2)

    num = (sw * sxy - sx * sy)
    denom = (sw * sx2 - sx ** 2)

    if denom == 0:
        b = 0
    else:
        b = num / denom

    a = (sy - b * sx) / (sw + eps)

    return a, b


@jit(nopython=True)
def _calc_linreg_wrapper_a(xyw):
    n = int(np.round(xyw.shape[0] / 3))
    xyw = xyw.reshape(3, n)

    a, b = _calc_linreg(xyw[0, :], xyw[1, :], xyw[2, :])

    return a


@jit(nopython=True)
def _calc_linreg_wrapper_b(xyw):
    n = int(np.round(xyw.shape[0] / 3))
    xyw = xyw.reshape(3, n)

    a, b = _calc_linreg(xyw[0, :], xyw[1, :], xyw[2, :])

    return b


def _piecewise_linreg(xyw, window_width=3):
    n = int(np.round((window_width - 1) / 2))

    piece_a = generic_filter(xyw.T, _calc_linreg_wrapper_a, size=(3, window_width), mode='nearest')
    piece_b = generic_filter(xyw.T, _calc_linreg_wrapper_b, size=(3, window_width), mode='nearest')

    # pad array
    piece_a = np.pad(piece_a[1, :], n, 'edge')
    piece_b = np.pad(piece_b[1, :], n, 'edge')

    smooth_a = np.convolve(piece_a, np.ones(window_width) / window_width, mode='valid')
    smooth_b = np.convolve(piece_b, np.ones(window_width) / window_width, mode='valid')

    y_est = smooth_b * xyw[:, 0] + smooth_a

    return y_est


# Apply the swets filter on 1d array
def _apply_swets1d(y):
    window_width = 3  # window width

    # dont smooth if all nan
    if np.all(np.isnan(y)):
        return y

    y_smoothed = np.zeros_like(y)
    y_smoothed[:] = np.nan

    x = np.flatnonzero(~np.isnan(y))
    w = _calc_weights(y[x])
    xyw = np.stack((x, y[x], w), axis=1)
    y_smoothed[x] = _piecewise_linreg(xyw, window_width)

    return y_smoothed

# slow: 3024x3024x7 array takes approx 1 hour on my machine
def swets_filter(data, do_interpolate=True, axis=2, invert=False):
    """
    :param data: np.array((y,x,t))
    NDVI raster timeseries. Each image in timeseries is shape y,x. Images are stacked along t (time dimension)
    nan values are initially ignored by swets. At the last step, nan values are replaced by linear interpolation
    :param do_interpolate: bool
    True if we want to apply interpolation of nan values
    :param axis: int
    Axis for the time-dimension in the array. Filtering/interpolation will be apply along this axis
    :param invert: bool
    Inversion of the data and output. This is useful for albedo where bad clous maksing will force the values up,
    instead of down in NDVI.

    :return: y_smoothed: np.array(y,x,t)
    """

    if invert:
        data = data * -1

    print('running filter...')
    y_smoothed = np.apply_along_axis(_apply_swets1d, axis, data)

    if do_interpolate:
        print('running interpolation...')
        y_smoothed = np.apply_along_axis(_interpolate1d, axis, y_smoothed)

    if invert:
        y_smoothed = y_smoothed * -1

    return y_smoothed.astype(data.dtype)



# -*- coding: utf-8 -*-
"""
Created on Wed Oct  7 11:38:16 2020

@author: rmgu
"""

import numpy as np

# Based on https://www.spiedigitallibrary.org/journals/journal-of-applied-remote-sensing/volume-9/issue-1/096095/Spatiotemporal-image-fusion-model-for-enhancing-the-temporal-resolution-of/10.1117/1.JRS.9.096095.full?SSO=1
# Spatiotemporal image-fusion model for enhancing the temporal resolution of Landsat-8 surface
# reflectance images using MODIS images

# To be run on 10-day MODIS NDVI composites nad Landsat images
# TO DO
# 1. Read images
# 2. Resample MODIS to Landsat projection, extent and resolution
# 3. Run predict_image
# 4. Save the output


# Section 3
def predict_image(hr_image_t1, lr_image_t1, lr_image_t2):

    rate_of_change = lr_image_t2 / lr_image_t1
    change = lr_image_t2 - lr_image_t1

    negligible_change = np.abs(change) <= 0.15 * rate_of_change
    negative_change = np.logical_and(change < 0, np.abs(change) > 0.15 * rate_of_change)
    positive_change = np.logical_and(change > 0, np.abs(change) > 0.15 * rate_of_change)

    negligible_change_func = np.poly1d(np.polyfit(lr_image_t1[negligible_change],
                                                  lr_image_t2[negligible_change],
                                                  1))
    negative_change_func = np.poly1d(np.polyfit(lr_image_t1[negative_change],
                                                lr_image_t2[negative_change],
                                                1))
    positive_change_func = np.poly1d(np.polyfit(lr_image_t1[positive_change],
                                                lr_image_t2[positive_change],
                                                1))

    predicted_image = np.empty(hr_image_t1.shape)
    predicted_image[negligible_change] = negligible_change_func(hr_image_t1[negligible_change])
    predicted_image[negative_change] = negative_change_func(hr_image_t1[negative_change])
    predicted_image[positive_change] = positive_change_func(hr_image_t1[positive_change])

    return predicted_image



import numpy as np
from pywapor.et_look_v2 import constants as con
import warnings


def longitude_rad(lon_deg):
    r"""
    Converts longitude from degrees to radians.

    Parameters
    ----------
    lon_deg : float
        longitude in degrees, 
        :math:`\phi`
        [deg]

    Returns
    -------
    lon : float
        longitude, 
        :math:`\phi`
        [rad]

    """
    return lon_deg * np.pi/180.0


def latitude_rad(lat_deg):
    r"""
    Converts latitude from degrees to radians.

    Parameters
    ----------
    lat_deg : float
        latitude in degrees, 
        :math:`\lambda`
        [deg]

    Returns
    -------
    lat : float
        latitude, 
        :math:`\lambda`
        [rad]

    """
    return lat_deg * np.pi/180.0


def slope_rad(slope_deg):
    r"""
    Converts slope from degrees to radians.

    Parameters
    ----------
    slope_deg : float
        slope in degrees, 
        :math:`s`
        [deg]

    Returns
    -------
    slope : float
        slope, 
        :math:`\Delta`
        [rad]

    """
    return slope_deg * np.pi/180.0


def aspect_rad(aspect_deg):
    r"""
    Converts aspect from degrees to radians.

    Parameters
    ----------
    aspect_deg : float
        aspect in degrees, 
        :math:`s`
        [deg]

    Returns
    -------
    aspect : float
        aspect (0 is north; pi is south), 
        :math:`\alpha`
        [rad]
    """
    return aspect_deg * np.pi/180.0


def declination(doy):
    r"""
    Computes the solar declination which is the angular height of the sun
    above the astronomical equatorial plane in radians.

    .. math ::

        \delta=0.409 \cdot \sin\left(\frac{2\pi \cdot J}{365}-1.39\right)

    Parameters
    ----------
    doy : float
        julian day of the year, 
        :math:`J`
        [-]

    Returns
    -------
    decl : float
        declination, 
        :math:`\delta`
        [rad]

    Examples
    --------
    >>> import ETLook.solar_radiation as solrad
    >>> solrad.declination(180)
    0.40512512455439242
    """
    return 0.409 * np.sin(2 * np.pi / 365 * doy - 1.39)


def earth_sun_distance(doy):
    r"""
    Computes the earth sun distance in Angstrom Unit where 1 AU is 1.496e8 km.

    .. math ::

        d_{r}=1+0.033 \cdot \cos\left(\frac{2\pi \cdot J}{365}\right)

    Parameters
    ----------
    doy : float
        julian day of the year, 
        :math:`J`
        [-]

    Returns
    -------
    ed : float
        earth sun distance, 
        :math:`d_{r}`
        [AU]

    Examples
    --------
    >>> import ETLook.solar_radiation as solrad
    >>> solrad.earth_sun_distance(180)
    0.96703055420162642
    """

    warnings.warn(('Using "ed" for the inverse earth-sun distance is deprecated, '
                   'use "iesd" (solar_radiation.inverse_earth_sun_distance) instead'),
                  DeprecationWarning, stacklevel=2)

    return 1 + 0.033 * np.cos(doy * 2 * np.pi / 365.0)


def inverse_earth_sun_distance(doy):
    r"""
    Computes the inverse earth sun distance (iesd) in Angstrom Unit where 1 AU is 1.496e8 km.

    .. math ::

        d_{r}=1+0.033 \cdot \cos\left(\frac{2\pi \cdot J}{365}\right)

    Parameters
    ----------
    doy : float
        julian day of the year, 
        :math:`J`
        [-]

    Returns
    -------
    iesd : float
        inverse earth sun distance, 
        :math:`d_{r}`
        [AU]

    Examples
    --------
    >>> import ETLook.solar_radiation as solrad
    >>> solrad.inverse_earth_sun_distance(180)
    0.96703055420162642
    """

    return 1 + 0.033 * np.cos(doy * 2 * np.pi / 365.0)


def actual_earth_sun_distance(iesd):
    r"""
    Computes the earth sun distance (esd) in Angstrom Unit where 1 AU is 1.496e8 km.

    .. math ::

        d_{r}=1+0.033 \cdot \cos\left(\frac{2\pi \cdot J}{365}\right)

    Parameters
    ----------
    iesd : float
        inverse earth sun distance, 
        :math:`J`
        [AU]

    Returns
    -------
    esd : float
        earth sun distance, 
        :math:`d_{r}`
        [AU]

    Examples
    --------
    >>> import ETLook.solar_radiation as solrad
    >>> solrad.actual_earth_sun_distance(180)
    1.034093489244084
    """

    return 1 / iesd


def seasonal_correction(doy):
    r"""
    Computes the seasonal correction for solar time  in hours.

    .. math ::

        b=\frac{2\pi\left(J-81\right)}{365}

    .. math ::

        s_{c}= 0.1645 \cdot sin \left( 2b \right) - 0.1255 \cdot cos \left(b \right) - 0.025 \left(b\right)

    Parameters
    ----------
    doy : float
        julian day of the year, 
        :math:`J`
        [-]

    Returns
    -------
    sc : float
        seasonal correction, 
        :math:`s_{c}`
        [hours]

    Examples
    --------
    >>> import ETLook.solar_radiation as solrad
    >>> solrad.seasonal_correction(180)
    -0.052343379605521212
    """
    b = 2 * np.pi * (doy - 81) / 365.0
    return 0.1645 * np.sin(2 * b) - 0.1255 * np.cos(b) - 0.025 * np.sin(b)


def sunset_hour_angle(lat, decl):
    r"""
    Computes the sunset hour angle.

    .. math ::

        w_{s}=\arccos(-\tan(\lambda)\cdot \tan(\delta))

    Parameters
    ----------
    decl : float
        solar declination, 
        :math:`\delta`
        [rad]
    lat : float
        latitude, 
        :math:`\lambda`
        [rad]

    Returns
    -------
    ws : float
        sunset hour angle, 
        :math:`w_{s}`
        [rad]

    """
    return np.arccos(-(np.tan(lat) * np.tan(decl)))


def hour_angle(sc, dtime, lon=0):
    r"""
    Computes the hour angle which is zero at noon and -pi at 0:00 am and
    pi at 12:00 pm

    .. math ::

        \omega=\left(\frac{\pi}{12}\right)\cdot \left(t+s_{c}-12\right)

    Parameters
    ----------
    sc : float
        seasonal correction, 
        :math:`s_{c}`
        [hours]
    dtime : float
        decimal time, 
        :math:`t`
        [hours]
    lon : float
        longitude, 
        :math:`\phi`
        [rad]

    Returns
    -------
    ha : float
        hour_angle, 
        :math:`\omega`
        [rad]

    Examples
    --------
    >>> import ETLook.solar_radiation as solrad
    >>> solrad.hour_angle(sc=solrad.seasonal_correction(75), dtime=11.4)
    -0.19793970172084141
    """
    dtime = dtime + (lon / (15*np.pi/180.0))
    return (np.pi / 12.0) * (dtime + sc - 12.0)


def inst_solar_radiation_toa(csza, iesd):
    r"""
    Computes the instantaneous solar radiation at the top of
    the atmosphere [Wm-2].

    .. math ::

        S_{toa}^{i} = S_{sun} \cdot d_{r} \cdot \phi

    Parameters
    ----------
    csza : float
        cosine solar zenith angle, 
        :math:`\phi`
        [-]
    iesd : float
        inverse earth sun distance, 
        :math:`d_{r}`
        [AU]

    Returns
    -------
    ra_i_toa : float
        instantaneous solar radiation at top of atmosphere, 
        :math:`S_{toa}^{i}`
        [Wm-2]

    Examples
    --------
    >>> import ETLook.solar_radiation as solrad
    >>> doy = 1
    >>> sc = solrad.seasonal_correction(doy)
    >>> ha = solrad.hour_angle(sc, dtime=12)
    >>> decl = solrad.declination(doy)
    >>> csza = solrad.cosine_solar_zenith_angle(ha, decl, 0)
    >>> iesd = solrad.inverse_earth_sun_distance(doy)
    >>> solrad.inst_solar_radiation_toa(csza, iesd)
    1299.9181944414036
    """
    return csza * con.sol * iesd


def daily_solar_radiation_toa(sc, decl, iesd, lat, slope=0, aspect=0):
    r"""
    Computes the daily solar radiation at the top of the atmosphere.

    .. math ::

        S_{toa}=S_{sun} \cdot d_{r}\int_{i=-\pi}^{i=\pi}S_{toa}^{i}

    Parameters
    ----------
    iesd : float
        inverse earth sun distance, 
        :math:`d_{r}`
        [AU]
    decl : float
        solar declination, 
        :math:`\delta`
        [rad]
    sc : float
        seasonal correction, 
        :math:`s_{c}`
        [hours]
    lat : float
        latitude, 
        :math:`\lambda`
        [rad]
    slope : float
        slope, 
        :math:`\Delta`
        [rad]
    aspect : float
        aspect (0 is north; pi is south), 
        :math:`\alpha`
        [rad]

    Returns
    -------
    ra_24_toa : float
        daily solar radiation at the top of atmosphere, 
        :math:`S_{toa}`
        [Wm-2]

    Examples
    --------
    >>> import ETLook.solar_radiation as solrad
    >>> from math import pi
    >>> doy = 1
    >>> sc = solrad.seasonal_correction(doy)
    >>> decl = solrad.declination(doy)
    >>> iesd = solrad.inverse_earth_sun_distance(doy)
    >>> solrad.daily_solar_radiation_toa(sc, decl, iesd, lat=25*pi/180.0)
    265.74072308978026
    """

    # hour angle for the whole day in half-hourly intervals (0:15-23:45)
    t_start = 0.25
    t_end = 24.00
    interval = 0.5
    times = [t_start+i*interval for i in range(0, 48)]
    hours = [hour_angle(sc, t) for t in times]

    ra24 = 0

    for t in hours:
        csza = cosine_solar_zenith_angle(t, decl, lat, slope, aspect)
        ra24 += inst_solar_radiation_toa(csza, iesd) / len(hours)

    # return the average daily radiation
    return ra24


def cosine_solar_zenith_angle(ha, decl, lat, slope=0, aspect=0):
    r"""
    computes the cosine of the solar zenith angle [-].

    .. math ::

        \phi = & \sin\left(\delta\right) \cdot \sin\left(\lambda\right) \cdot  \cos\left(\Delta\right) - \\
        & \sin\left(\delta\right) \cdot \cos\left(\lambda\right) \cdot \sin\left(\Delta\right) + \\
        & \cos\left(\delta\right) \cdot \cos\left(\lambda\right) \cdot \cos\left(\Delta\right) \cdot \cos\left(\omega\right)+\\
        & \cos\left(\delta\right) \cdot \sin\left(\lambda\right) \cdot \sin\left(\Delta\right) \cdot \sin\left(\alpha\right) \cdot \cos\left(\omega\right)+\\
        & \cos\left(\delta\right) \cdot \sin\left(\Delta\right) \cdot  \sin\left(\alpha\right) \cdot \sin\left(\omega\right)

    Parameters
    ----------
    ha : float
        hour angle, 
        :math:`\omega`
        [rad]
    decl : float
        declination, 
        :math:`\delta`
        [rad]
    lat : float
        latitude, 
        :math:`\lambda`
        [rad]
    slope : float
        slope, 
        :math:`\Delta`
        [rad]
    aspect : float
        aspect (0 is north; pi is south), 
        :math:`\alpha`
        [rad]

    Returns
    -------
    csza : float
        cosine solar zenith angle, 
        :math:`\phi`
        [-]

    Examples
    --------
    >>> import ETLook.solar_radiation as solrad
    >>> sc = solrad.seasonal_correction(1)
    >>> ha = solrad.hour_angle(sc, dtime=12)
    >>> solrad.cosine_solar_zenith_angle(ha, decl=solrad.declination(1), lat=0)
    0.92055394167363314
    """
    t1 = np.sin(decl) * np.sin(lat) * np.cos(slope)
    t2 = np.sin(decl) * np.cos(lat) * np.sin(slope) * np.cos(aspect - np.pi)
    t3 = np.cos(decl) * np.cos(lat) * np.cos(slope)
    t4 = np.cos(decl) * np.sin(lat) * np.sin(slope) * np.cos(aspect - np.pi)
    t5 = np.cos(decl) * np.sin(slope) * np.sin(aspect - np.pi)
    csza = t1 - t2 + t3 * np.cos(ha) + t4 * np.cos(ha) + t5 * np.sin(ha)

    # check if the sun is above the horizon
    check = np.sin(decl) * np.sin(lat) + np.cos(decl) * \
        np.cos(lat) * np.cos(ha)

    nans = np.logical_or(np.isnan(csza), np.isnan(check))

    res = np.where(np.logical_and(csza > 0, check >= 0), csza, 0)
    res[nans] = np.nan

    return res


def daily_solar_radiation_toa_flat(decl, iesd, lat, ws):
    r"""
    Computes the daily solar radiation at the top of the atmosphere for a flat
    surface.

    .. math ::

        S_{toa,f}=\frac{S_{sun}}{\pi} \cdot d_{inv,r} \cdot (w_{s} \cdot \sin(\lambda) \cdot \sin(\delta) +
                  \cos(\lambda)\cdot\cos(\delta)\cdot\sin(w_{s}))

    Parameters
    ----------
    decl : float
        solar declination, 
        :math:`\delta`
        [rad]
    iesd : float
        inverse earth sun distance, 
        :math:`d_{inv,r}`
        [AU]
    lat : float
        latitude, 
        :math:`\lambda`
        [rad]
    ws : float
        sunset hour angle, 
        :math:`w_{s}`
        [rad]

    Returns
    -------
    ra_24_toa_flat : float
        daily solar radiation at the top of atmosphere for a flat surface, 
        :math:`S_{toa,f}`
        [Wm-2]

    """
    ra_flat = (con.sol / np.pi) * iesd * (ws * np.sin(lat) * np.sin(decl) +
                                        np.cos(lat) * np.cos(decl) * np.sin(ws))

    return ra_flat


def daily_solar_radiation_flat(ra_24_toa_flat, trans_24):
    r"""
    Computes the daily solar radiation at the earth's surface.

    .. math ::

        S^{\downarrow} = \tau \cdot S_{toa}

    Parameters
    ----------
    ra_24_toa_flat : float
        daily solar radiation at the top of atmosphere for a flat surface, 
        :math:`S_{toa}`
        [Wm-2]
    trans_24 : float
        daily atmospheric transmissivity, 
        :math:`\tau`
        [-]

    Returns
    -------
    ra_24 : float
        daily solar radiation for a flat surface, 
        :math:`S^{\downarrow}`
        [Wm-2]

    """
    return ra_24_toa_flat * trans_24


def diffusion_index(trans_24, diffusion_slope=-1.33, diffusion_intercept=1.15):
    r"""
    Computes the diffusion index, the ratio between diffuse and direct
    solar radiation. The results are clipped between 0 and 1.

    .. math ::

        I_{diff} = a_{diff}+b_{diff} \cdot \tau

    Parameters
    ----------
    trans_24 : float
        daily atmospheric transmissivity, 
        :math:`\tau`
        [-]
    diffusion_slope : float
        slope of diffusion index vs transmissivity relationship, 
        :math:`b_{diff}`
        [-]
    diffusion_intercept : float
        intercept of diffusion index vs transmissivity relationship, 
        :math:`a_{diff}`
        [-]

    Returns
    -------
    diffusion_index : float
        diffusion_index, 
        :math:`I_{diff}`
        [-]

    """
    res = diffusion_intercept + trans_24 * diffusion_slope

    res = np.clip(res, 0, 1)

    return res


def daily_total_solar_radiation(ra_24_toa, ra_24_toa_flat, diffusion_index, trans_24):
    r"""
    Computes the daily solar radiation at the earth's surface taken
    diffuse and direct solar radiation into account.

    .. math ::

        S^{\downarrow} = I_{diff} \cdot \tau \cdot S_{toa,f} +(1-I_{diff}) \cdot \tau \cdot S_{toa}

    Parameters
    ----------
    ra_24_toa : float
        daily solar radiation at the top of atmosphere, 
        :math:`S_{toa}`
        [Wm-2]
    ra_24_toa_flat : float
        daily solar radiation at the top of atmosphere for a flat surface, 
        :math:`S_{toa,f}`
        [Wm-2]
    diffusion_index : float
        diffusion_index, 
        :math:`I_{diff}`
        [-]
    trans_24 : float
        daily atmospheric transmissivity, 
        :math:`\tau`
        [-]

    Returns
    -------
    ra_24 : float
        daily solar radiation, 
        :math:`S^{\downarrow}`
        [Wm-2]

    """
    diffuse = trans_24 * ra_24_toa_flat * diffusion_index
    direct = trans_24 * ra_24_toa * (1 - diffusion_index)
    return diffuse + direct

def daily_solar_radiation_toa_new(sc, decl, iesd, lat, doy, slope=0, aspect=0):
    r"""
    Computes the daily solar radiation at the top of the atmosphere.

    .. math ::

        S_{toa}=S_{sun} \cdot d_{r} \cdot \int_{i=-\pi}^{i=\pi}S_{toa}^{i}

    Parameters
    ----------
    iesd : float
        inverse earth sun distance, 
        :math:`d_{r}`
        [AU]
    decl : float
        solar declination, 
        :math:`\delta`
        [rad]
    sc : float
        seasonal correction, 
        :math:`s_{c}`
        [hours]
    lat : float
        latitude, 
        :math:`\lambda`
        [rad]
    slope : float
        slope, 
        :math:`\Delta`
        [rad]
    aspect : float
        aspect (0 is north; pi is south), 
        :math:`\alpha`
        [rad]

    Returns
    -------
    ra_24_toa : float
        daily solar radiation at the top of atmosphere, 
        :math:`S_{toa}`
        [Wm-2]

    Examples
    --------
    >>> import ETLook.solar_radiation as solrad
    >>> from math import pi
    >>> doy = 1
    >>> sc = solrad.seasonal_correction(doy)
    >>> decl = solrad.declination(doy)
    >>> iesd = solrad.inverse_earth_sun_distance(doy)
    >>> solrad.daily_solar_radiation_toa(sc, decl, iesd, lat=25*pi/180.0)
    265.74072308978026
    """
    #print(type(slope))
    if type(slope)==int:
        slope = np.zeros(lat.shape)
    #print(type(aspect))    
    if type(aspect)==int:
        aspect = np.zeros(lat.shape)
        
    gamma = np.deg2rad(np.rad2deg(aspect)-180)                               # Surface aspect angle (radians)
    a,b,c = Constants(decl,slope,gamma,lat)

    ra24 = np.zeros(np.shape(lat))*np.nan
    dr = 1 + 0.033 * np.cos(doy*2*np.pi/365)  # Inverse relative distance Earth-Sun
    constant=con.sol*dr/(2*np.pi)
    TwoPeriod= TwoPeriods(decl,slope,lat)  # all input in radians

    #2.) calculate the 24-hours extraterrestrial radiation (2 periods)
    ID = np.where(np.ravel(TwoPeriod==True))
    ra24.flat[ID]=TwoPeriodSun(constant, decl, slope.flat[ID], gamma.flat[ID], lat.flat[ID])

    #3.) calculate the 24-hours extraterrestrial radiation (1 period)
    ID = np.where(np.ravel(TwoPeriod==False))
    ra24.flat[ID]=OnePeriodSun(constant, decl, slope.flat[ID], gamma.flat[ID], lat.flat[ID])

    # Horizontal surface
    ws = np.arccos(-np.tan(decl) * np.tan(lat))  # Sunrise/sunset time angle

    # Extraterrestial radiation for a horizontal surface for 24-h period:
    Ra_hor_24 = (con.sol * dr / np.pi * (np.sin(decl) * np.sin(lat) * ws + np.cos(decl) * np.cos(lat) * np.sin(ws)))
    # cos_theta_flat = (np.sin(delta) * np.sin(phi) + np.cos(delta) * np.cos(phi) * np.cos(w))

    # Mountain radiation
    ra24 = np.where(ra24 > 0.1 * Ra_hor_24, ra24 / np.cos(slope),
                           Ra_hor_24)
    ra24[ra24 > 600.0] = 600.0
    
    return ra24

def OnePeriodSun(constant,delta,s,gamma,phi):
    '''
    Based on :footcite:t:`tasumi2006based`, 
    calculates the 24-hours extraterrestrial radiation when there is one sun period.
    '''
    sunrise,sunset = SunHours(delta,s,gamma,phi)
    Vals=IntegrateSlope(constant,sunrise,sunset,delta,s,gamma,phi)

    return(Vals)

def TwoPeriodSun(constant,delta,s,gamma,phi):
    '''
    Based on :footcite:t:`tasumi2006based`, 
    calculates the 24-hours extraterrestrial radiation when there are two sun period.
    '''
    A1, A2 = SunHours(delta,s,gamma,phi)
    a,b,c = Constants(delta,s,gamma,phi)
    riseSlope, setSlope = BoundsSlope(a,b,c)
    B1 = np.maximum(riseSlope,setSlope)
    B2 = np.minimum(riseSlope,setSlope)
    Angle_B1 = AngleSlope(a,b,c,B1)
    Angle_B2 = AngleSlope(a,b,c,B2)

    B1[abs(Angle_B1) > 0.001] = np.pi - B1[abs(Angle_B1) > 0.001]
    B2[abs(Angle_B2) > 0.001] = -np.pi - B2[abs(Angle_B2) > 0.001]

    # Check if two periods really exist
    ID = np.ravel_multi_index(np.where(np.logical_and(B2 >= A1, B1 >= A2) == True),a.shape)
    Val = IntegrateSlope(constant,B2.flat[ID],B1.flat[ID],delta,s.flat[ID],gamma.flat[ID],phi.flat[ID])
    ID = ID[Val < 0]

    # Finally calculate resulting values
    Vals = np.zeros(B1.shape)

    Vals.flat[ID] = (IntegrateSlope(constant,A1.flat[ID],B2.flat[ID],delta,s.flat[ID],gamma.flat[ID],phi.flat[ID])  +
                   IntegrateSlope(constant,B1.flat[ID],A2.flat[ID],delta,s.flat[ID],gamma.flat[ID],phi.flat[ID]))
    ID = np.ravel_multi_index(np.where(Vals == 0),a.shape)
    Vals.flat[ID] = IntegrateSlope(constant,A1.flat[ID],A2.flat[ID],delta,s.flat[ID],gamma.flat[ID],phi.flat[ID])

    return(Vals)

#------------------------------------------------------------------------------
def IntegrateSlope(constant,sunrise,sunset,delta,s,gamma,phi):
    '''
    Based on :footcite:t:`tasumi2006based`, Equation 5,
    calculates the 24 hours extraterrestrial radiation.
    '''
    # correct the sunset and sunrise angels for days that have no sunset or no sunrise
    SunOrNoSun = np.logical_or(((np.abs(delta + phi)) > (np.pi/2)),((np.abs(delta - phi)) > (np.pi/2)))
    integral=np.zeros(s.shape)
    ID = np.where(np.ravel(SunOrNoSun==True))

    # No sunset
    IDNoSunset = np.where(np.ravel(abs(delta+phi.flat[ID])>(np.pi/2)))
    if np.any(IDNoSunset) == True:
        sunset1=np.pi
        sunrise1=-np.pi
        integral.flat[IDNoSunset] = constant * (np.sin(delta)*np.sin(phi)*np.cos(s)*(sunset1-sunrise1)
            - np.sin(delta)*np.cos(phi)*np.sin(s)*np.cos(gamma)*(sunset1-sunrise1)
            + np.cos(delta)*np.cos(phi)*np.cos(s)*(np.sin(sunset1)-np.sin(sunrise1))
            + np.cos(delta)*np.sin(phi)*np.sin(s)*np.cos(gamma)*(np.sin(sunset1)-np.sin(sunrise1))
            - np.cos(delta)*np.sin(s)*np.sin(gamma)*(np.cos(sunset1)-np.cos(sunrise1)))

    # No sunrise
    elif np.any(IDNoSunset) == False:
        integral.flat[IDNoSunset==False]=constant * (np.sin(delta)*np.sin(phi)*np.cos(s)*(0)
            - np.sin(delta)*np.cos(phi)*np.sin(s)*np.cos(gamma)*(0)
            + np.cos(delta)*np.cos(phi)*np.cos(s)*(np.sin(0)-np.sin(0))
            + np.cos(delta)*np.sin(phi)*np.sin(s)*np.cos(gamma)*(np.sin(0)-np.sin(0))
            - np.cos(delta)*np.sin(s)*np.sin(gamma)*(np.cos(0)-np.cos(0)))

    ID = np.where(np.ravel(SunOrNoSun==False))
    integral.flat[ID] = constant * (np.sin(delta)*np.sin(phi)*np.cos(s)*(sunset-sunrise)
            - np.sin(delta)*np.cos(phi)*np.sin(s)*np.cos(gamma)*(sunset-sunrise)
            + np.cos(delta)*np.cos(phi)*np.cos(s)*(np.sin(sunset)-np.sin(sunrise))
            + np.cos(delta)*np.sin(phi)*np.sin(s)*np.cos(gamma)*(np.sin(sunset)-np.sin(sunrise))
            - np.cos(delta)*np.sin(s)*np.sin(gamma)*(np.cos(sunset)-np.cos(sunrise)))

    return(integral)

def BoundsHorizontal(delta,phi):
    '''
    Based on :footcite:t:`tasumi2006based`, 
    this function calculates sunrise hours based on earth inclination and latitude.
    If there is no sunset or sunrise hours the values are either set to 0 (polar night) or pi (polar day).
    '''
    bound = np.arccos(-np.tan(delta)*np.tan(phi))
    bound[abs(delta+phi) > np.pi/2] = np.pi
    bound[abs(delta-phi) > np.pi/2] = 0

    return(bound)

def TwoPeriods(delta,s,phi):
    '''
    Based on :footcite:t:`tasumi2006based`, 
    creates a boolean map with True values for places with two sunsets.
    '''
    TwoPeriods = (np.sin(s) > np.ones(phi.shape)*np.sin(phi)*np.cos(delta)+np.cos(phi)*np.sin(delta))

    return(TwoPeriods)

def SunHours(delta,slope,slopedir,lat):
    # Define sun hours in case of one sunlight period

    a,b,c = Constants(delta,slope,slopedir,lat)
    riseSlope, setSlope = BoundsSlope(a,b,c)
    bound = BoundsHorizontal(delta,lat)

    Calculated = np.zeros(slope.shape, dtype = bool)
    RiseFinal = np.zeros(slope.shape)
    SetFinal = np.zeros(slope.shape)

    # First check sunrise is not nan
    # This means that their is either no sunrise (whole day night) or no sunset (whole day light)
    # For whole day light, use the horizontal sunrise and whole day night a zero..
    Angle4 = AngleSlope(a,b,c,-bound)
    RiseFinal[np.logical_and(np.isnan(riseSlope),Angle4 >= 0.0)] = -bound[np.logical_and(np.isnan(riseSlope),Angle4 >= 0.0)]
    Calculated[np.isnan(riseSlope)] = True

    # Step 1 > 4
    Angle1 = AngleSlope(a,b,c,riseSlope)
    Angle2 = AngleSlope(a,b,c,-bound)

    ID = np.ravel_multi_index(np.where(np.logical_and(np.logical_and(Angle2 < Angle1+0.001 ,Angle1 < 0.001),Calculated == False) == True),a.shape)
    RiseFinal.flat[ID] = riseSlope.flat[ID]
    Calculated.flat[ID] = True
    # step 5 > 7
    Angle3 = AngleSlope(a,b,c,-np.pi - riseSlope)

    ID = np.ravel_multi_index(np.where(np.logical_and(np.logical_and(-bound<(-np.pi-riseSlope),Angle3 <= 0.001),Calculated == False) == True),a.shape)
    RiseFinal.flat[ID] = -np.pi -riseSlope.flat[ID]
    Calculated.flat[ID] = True

    # For all other values we use the horizontal sunset if it is positive, otherwise keep a zero
    RiseFinal[Calculated == False] = -bound[Calculated == False]

    # Then check sunset is not nan or < 0
    Calculated = np.zeros(slope.shape, dtype = bool)

    Angle4 = AngleSlope(a,b,c,bound)
    SetFinal[np.logical_and(np.isnan(setSlope),Angle4 >= 0.0)] = bound[np.logical_and(np.isnan(setSlope),Angle4 >= 0.0)]
    Calculated[np.isnan(setSlope)] = True

    # Step 1 > 4
    Angle1 = AngleSlope(a,b,c,setSlope)
    Angle2 = AngleSlope(a,b,c,bound)

    ID = np.ravel_multi_index(np.where(np.logical_and(np.logical_and(Angle2 < Angle1+0.001,Angle1 < 0.001),Calculated == False) == True),a.shape)
    SetFinal.flat[ID] = setSlope.flat[ID]
    Calculated.flat[ID] = True
    # step 5 > 7
    Angle3 = AngleSlope(a,b,c,np.pi - setSlope)

    ID = np.ravel_multi_index(np.where(np.logical_and(np.logical_and(bound>(np.pi-setSlope),Angle3 <= 0.001),Calculated == False) == True),a.shape)
    SetFinal.flat[ID] = np.pi - setSlope.flat[ID]
    Calculated.flat[ID] = True

    # For all other values we use the horizontal sunset if it is positive, otherwise keep a zero
    SetFinal[Calculated == False] = bound[Calculated == False]

    #    Angle4 = AngleSlope(a,b,c,bound)
    #    SetFinal[np.logical_and(Calculated == False,Angle4 >= 0)] = bound[np.logical_and(Calculated == False,Angle4 >= 0)]

    # If Sunrise is after Sunset there is no sunlight during the day
    SetFinal[SetFinal <= RiseFinal] = 0.0
    RiseFinal[SetFinal <= RiseFinal] = 0.0

    return(RiseFinal,SetFinal)

def AngleSlope(a,b,c,w):
    '''
    Based on :footcite:t:`tasumi2006based`, 
    calculates the cos zenith angle by using the hour angle and constants.
    '''
    angle = -a + b*np.cos(w) + c*np.sin(w)

    return(angle)

def BoundsSlope(a,b,c):
    '''
    Based on :footcite:t:`tasumi2006based`,  Equation 13,
    this function calculates candidate values for sunrise and sunset hour angles.
    '''
    Div = (b**2+c**2)
    Div[Div <= 0] = 0.00001
    sinB = (a*c + b*np.sqrt(b**2+c**2-a**2)) / Div
    sinA = (a*c - b*np.sqrt(b**2+c**2-a**2)) / Div

    sinB[sinB < -1] = -1; sinB[sinB > 1] = 1    # Limits see appendix A.2.i
    sinA[sinA < -1] = -1; sinA[sinA > 1] = 1    # Limits see appendix A.2.i

    sunrise = np.arcsin(sinA)
    sunset = np.arcsin(sinB)

    return(sunrise,sunset)

def Constants(delta,s,gamma,phi):
    '''
    Based on :footcite:t:`tasumi2006based`,  Equation 11,
    determines constants for calculating the exterrestial solar radiation.
    '''
    a = np.sin(delta)*np.cos(phi)*np.sin(s)*np.cos(gamma) - np.sin(delta)*np.sin(phi)*np.cos(s)
    b = np.cos(delta)*np.cos(phi)*np.cos(s) + np.cos(delta)*np.sin(phi)*np.sin(s)*np.cos(gamma)
    c = np.cos(delta)*np.sin(s)*np.sin(gamma)

    return(a,b,c)    
    



import pywapor.general.processing_functions as PF
import numpy as np
import pywapor

def lapse_rate_temperature(tair_file, dem_file):

    # import matplotlib.pyplot as plt

    # tair_file = r"/Volumes/Data/pre_et_look_NEW/RAW/MERRA/Air_Temperature/daily_MERRA2/t2m_MERRA_K_daily_2019.07.06.tif"
    # tair_file = r"/Volumes/Data/pre_et_look_NEW/RAW/GEOS/Air_Temperature/daily/t2m_GEOS_K_daily_2019.07.06.tif"

    # def _plot_array(array):
    #     plt.clf()
    #     plt.imshow(array)
    #     plt.title(array.shape)
    #     plt.colorbar()
    #     plt.show()

    ## 1
    ds_t_down = PF.reproject_dataset_example(tair_file, dem_file, 2)
    tempe = PF.open_as_array(ds_t_down)
    # _plot_array(tempe - 273.15)

    ## 2
    dem_down = PF.open_as_array(dem_file)
    # _plot_array(dem_down)

    ## 3
    ds_dem_up = PF.reproject_dataset_example(dem_file, tair_file, 4)
    
    dem_up = PF.open_as_array(ds_dem_up)
    dem_up[np.isnan(dem_up)] = 0.
    ds_dem_up = PF.Save_as_MEM(dem_up, ds_dem_up.GetGeoTransform(), PF.Get_epsg(ds_dem_up))
    
    ds_dem_up_down = PF.reproject_dataset_example(ds_dem_up, dem_file, 2)
    dem_up_ave = PF.open_as_array(ds_dem_up_down)
    # _plot_array(dem_up_ave)

    ## Correct wrong values
    dem_down[dem_down <= 0] = 0
    dem_up_ave[dem_up_ave <= 0] = 0

    tdown = pywapor.et_look_v2.meteo.disaggregate_air_temperature(tempe, dem_down, dem_up_ave, lapse = pywapor.et_look_v2.constants.lapse)
    # _plot_array(tdown)

    fh = tair_file.replace("_K_", "_C_")
    geo, projection = PF.get_geoinfo(dem_file)[:2]

    PF.Save_as_tiff(fh, tdown, geo, projection)

    # test_tair = r"/Volumes/Data/pre_et_look_ORIGINAL/ETLook_input_MODIS/20190706/tair_24_20190706.tif"
    # tempe_test = PF.open_as_array(test_tair)
    # _plot_array(tempe_test - tdown)

    return fh


# -*- coding: utf-8 -*-
import sys
from pywapor.collect.MERRA.DataAccess import DownloadData
from pywapor.collect.MERRA.DataAccess import VariablesInfo
import pywapor

def main(Dir, latlim, lonlim, Startdate, Enddate, Vars, Waitbar = 1, data_type = ["mean"]):
    """
    This function downloads MERRA daily data for a given variable, time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Vars -- ['t2m', 'v2m'] 
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Waitbar -- 1 (Default) Will print a waitbar
    """
    all_files = dict()

    username, password = pywapor.collect.get_pw_un.get("NASA")

    for Var in Vars:

        if Waitbar == 1:
            print('\nDownloading daily MERRA %s data for the period %s till %s' %(Var, Startdate, Enddate))

        # Download data
        all_files = {**all_files, **DownloadData(Dir, Var, Startdate, Enddate, latlim, lonlim, "daily_MERRA2", '', username, password, Waitbar = 1, data_type = data_type)}

        TimeStep = "daily_MERRA2"
        VarInfo = VariablesInfo(TimeStep)
        Parameter = VarInfo.names[Var]

    return all_files

if __name__ == '__main__':
    main(sys.argv)



import sys
from pywapor.collect.Landsat.PreprocessLandsat import PreprocessLandsat


def main(data_dir, output_dir):

    PreprocessLandsat(data_dir, output_dir)

if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
import sys
from pywapor.collect.GEOS.DataAccess import DownloadData
import pandas as pd
import tqdm

def main(Dir, latlim, lonlim, Startdate, Enddate, Vars, Periods = [1,2,3,4,5,6,7,8], Waitbar = True):
    """
    This function downloads GEOS inst data for a given variable, time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Vars -- ['t2m', 'v2m'] 
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
	Periods -- [1,2,3,4,5,6,7,8] Period that needs to be downloaded. 1 period is 3 hour starting from noon
    Waitbar -- 1 (Default) Will print a waitbar
    """

    no_vars = len(Vars)
    no_periods = len(Periods)
    no_dates = len(pd.date_range(Startdate, Enddate, freq="D"))

    if isinstance(Waitbar, tqdm.tqdm):
        waitbar = Waitbar
    elif Waitbar:
        waitbar = tqdm.tqdm(total = no_vars * no_dates * no_periods, delay = 10)
    else:
        waitbar = False

    for Var in Vars:

        if Waitbar:
            waitbar.set_description_str(Var)

        for Period in Periods:
		
            # if Waitbar == 1:
            #     print('\nDownloading 3-hourly GEOS %s data for the period %s till %s, Period = %s' %(Var, Startdate, Enddate, Period))

            # Download data
            DownloadData(Dir, Var, Startdate, Enddate, latlim, lonlim, "three_hourly", Period, waitbar)

if __name__ == '__main__':
    main(sys.argv)



import os
import numpy as np
import xarray as xr
from datetime import datetime as dat
import pandas as pd
from dask.diagnostics import ProgressBar
import pywapor.post_et_look as pl
import pywapor.general.processing_functions as PF

def main(cmeta, dbs, epochs_info, temp_folder = None, example_ds = None,
                lean_output = True, diagnostics = None):
    """
    composite_type = "max", "mean", "min"
    temporal_interp = False, "linear", "nearest", "zero", "slinear", "quadratic", "cubic"
    spatial_interp = "nearest", "linear"
    """

    if isinstance(temp_folder, type(None)):
        fh = dbs[0][0]
        path = os.path.normpath(fh)
        parts = path.split(os.sep)
        temp_folder = os.path.join(*parts[:-4], "temporary")

    if not os.path.exists(temp_folder):
        os.makedirs(temp_folder)

    composite_type = cmeta["composite_type"]
    temporal_interp = cmeta["temporal_interp"]
    spatial_interp = cmeta["spatial_interp"]

    styling = { 1: ("*", "r", 1.0, "MOD13Q1"),
                2: ("o", "g", 1.0, "MYD13Q1"),
                3: ("v", "b", 1.0, "PROBAV"),
                4: ("s", "y", 1.0, "MCD43A3"),
                5: ("*", "purple", 1.0, "CHIRPS.v2.0"),
                6: ("p", "darkblue", 1.0, "GEOS"),
                7: ("h", "gray", 1.0, "MERRA"),
                999: ("P", "orange", 1.0, "-"),
                0: (".", "k", 0.7, "Interp.")}

    # Check if data is static
    if np.all([isinstance(composite_type, type(None)), 
                isinstance(temporal_interp, type(None)),
                len(dbs) == 1,
                len(dbs[0]) == 1]):
        ds = xr.open_dataset(dbs[0][0], engine="rasterio")
        ds = ds.rename_vars({"x": f"lon", "y": f"lat"})
        ds = ds.swap_dims({"x": f"lon", "y": f"lat"})
        ds = ds.interp_like(example_ds, method = "linear", kwargs={"fill_value": "extrapolate"},)
        ds = ds.rename({"band": "epoch"}).assign_coords({"epoch": [-9999]})
        ds = ds.rename({"band_data": "composite"})
        return ds

    def preprocess_func(ds):

        date_string = ds.encoding["source"].split("_")[-1]
        freq_string = ds.encoding["source"].split("_")[-2]
        if len(date_string.split(".")) > 4:
            date = dat.strptime(date_string, '%Y.%m.%d.%H.%M.tif')
        else:
            date = dat.strptime(date_string, '%Y.%m.%d.tif')
            t_offsets = {"daily": 12, "16-daily": 192, 
                        "daily-min":12, "daily-max":12, "5-daily": 60}
            date = date + pd.Timedelta(hours = t_offsets[freq_string])

        source = ds.encoding["source"].split("_")[-4]

        ds = ds.drop_vars("spatial_ref")
        ds = ds.squeeze("band")
        ds = ds.drop_vars("band")
        ds = ds.expand_dims("time", axis = 0)
        ds = ds.assign_coords({"time": [date]})
        ds = ds.rename_vars({"x": f"lon", "y": f"lat"})
        ds = ds.swap_dims({"x": f"lon", "y": f"lat"})

        sources = {v[3]: k for k, v in styling.items()}

        ds["band_data"] = ds.band_data.expand_dims("source")
        ds = ds.assign_coords({"source": [source]})

        ds["sources"] = np.isfinite(ds.band_data) * sources[source]

        return ds

    ds = None
    dss = list()
    dbs_names = list()

    # Open tif-files and apply spatial interpolation
    for db in dbs:

        check_geots(db)

        sub_ds = xr.open_mfdataset(db, concat_dim = "time", engine="rasterio", combine = "nested",
                                        preprocess = preprocess_func)
        dbs_names.append(sub_ds.source.values[0])

        if isinstance(example_ds, type(None)):
            example_ds = sub_ds.isel(time = 0, 
                                    source = 0).drop_vars(["source", "time"])
        else:
            sub_ds = sub_ds.interp_like(example_ds, method = spatial_interp, kwargs={"fill_value": "extrapolate"},)

        dss.append(sub_ds)

    ds = xr.merge(dss)

    for i, db_name in enumerate(dbs_names):
        if i == 0:
            ds_temp = ds.sel(source = db_name)
        else:
            # Combine datasets, this is where one ds is prioritised over another!
            ds_temp = ds_temp.fillna(ds.sel(source = db_name))

    ds = ds_temp

    fh = os.path.join(temp_folder, "intermediate.nc")

    ds = ds.reindex({"time": ds.time.sortby("time")})
    ds = calculate_ds(ds, fh, "--> Reprojecting datasets.").chunk({"time":-1})

    if temporal_interp:
        ds_resampled = ds.interpolate_na(dim="time")
        ds_resampled = ds_resampled.resample({"time": "12H"}).interpolate(temporal_interp).drop("sources")
        ds_resampled["sources"] = ds.sources
        ds_resampled["sources"] = ds_resampled["sources"].fillna(0.0)
        ds = ds_resampled

    if isinstance(epochs_info, type(None)):
        fh = os.path.join(temp_folder, f"{cmeta['var_name']}_ts.nc")
        ds = calculate_ds(ds, fh, "--> Exporting timeseries.")
        return ds

    epochs, epoch_starts, epoch_ends = epochs_info

    da = xr.DataArray(epochs, coords={"time":epoch_starts})
    ds["epochs"] = da.reindex_like(ds["time"], method = "ffill", tolerance = epoch_ends[-1] - epoch_starts[-1])

    if composite_type == "max":
        ds["composite"] = ds.band_data.groupby(ds["epochs"]).max().rename({"epochs": "epoch"})
    elif composite_type == "mean":
        ds["composite"] = ds.band_data.groupby(ds["epochs"]).mean().rename({"epochs": "epoch"})
    elif composite_type == "min":
        ds["composite"] = ds.band_data.groupby(ds["epochs"]).min().rename({"epochs": "epoch"})
    elif isinstance(composite_type, float):
        ds["composite"] = ds.band_data.groupby(ds["epochs"]).quantile(composite_type).rename({"epochs": "epoch"})
    else:
        print("No valid composite_type selected.")

    ds["epoch_starts"] = xr.DataArray(epoch_starts, coords = {"epoch": epochs})
    ds["epoch_ends"] = xr.DataArray(epoch_ends, coords = {"epoch": epochs})

    checklist = {True: dbs_names + ["Interp."], False: dbs_names}[True] # TODO interp data should not be included when not used
    sources_styling = {str(k): v for k, v in styling.items() if v[3] in checklist}
    ds.attrs = {str(k): str(v) for k, v in {**cmeta, **sources_styling}.items()}

    if diagnostics:
        graph_folder = diagnostics.pop("folder")
        ds_diags = ds.sel(lat = [v[0] for v in diagnostics.values()],
                          lon = [v[1] for v in diagnostics.values()], method = "nearest")
    else:
        ds_diags = None

    if lean_output: # TODO this option can be removed?
        ds = ds.drop_vars(["band_data", "sources", "epochs", "time"])

    fh = os.path.join(temp_folder, f"{cmeta['var_name']}_composite.nc")
    ds = calculate_ds(ds, fh, label = "--> Calculating composites.")

    if not isinstance(diagnostics, type(None)):
        fh = os.path.join(temp_folder, f"{cmeta['var_name']}_diags.nc")
        ds_diags = calculate_ds(ds_diags, fh, label = "--> Calculating diagnostics.")
        pl.plot_composite(ds_diags, diagnostics, graph_folder)
        diagnostics["folder"] = graph_folder

    return ds

def calculate_ds(ds, fh, label = None):
    while os.path.isfile(fh):
        fh = fh.replace(".nc", "_.nc")
    with ProgressBar(minimum = 30):
        if not isinstance(label, type(None)):
            print(label)
        ds.to_netcdf(fh)
    ds.close()
    ds = None
    ds = xr.open_dataset(fh)
    return ds

def check_geots(files):
    ref = PF.get_geoinfo(files[0])
    for fh in files[1:]:
        checker = PF.get_geoinfo(fh)
        assert ref == checker, f"ERROR: {files[0]} does not have same geotransform/projection as {fh}."

# if __name__ == "__main__":

#     import pywapor

#     project_folder = r"/Users/hmcoerver/On My Mac/pyWAPOR/example_data"
#     latlim = [28.9, 29.7]
#     lonlim = [30.2, 31.2]
#     startdate = "2021-06-01"
#     enddate = "2021-08-01"

#     epochs_info = pywapor.pre_et_look.create_dates(startdate, enddate, 6)

#     cmeta = {
#         "composite_type": "mean",
#         "temporal_interp": False,
#         "spatial_interp": "nearest",
#         "var_name": "ndvi",
#         "var_unit": "-",
#     }

#     dbs = [['/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MOD13/NDVI_MOD13Q1_-_16-daily_2021.06.26.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MOD13/NDVI_MOD13Q1_-_16-daily_2021.07.28.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MOD13/NDVI_MOD13Q1_-_16-daily_2021.07.12.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MOD13/NDVI_MOD13Q1_-_16-daily_2021.05.25.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MOD13/NDVI_MOD13Q1_-_16-daily_2021.08.13.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MOD13/NDVI_MOD13Q1_-_16-daily_2021.06.10.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MOD13/NDVI_MOD13Q1_-_16-daily_2021.08.29.tif'],
#     ['/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MYD13/NDVI_MYD13Q1_-_16-daily_2021.06.18.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MYD13/NDVI_MYD13Q1_-_16-daily_2021.08.21.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MYD13/NDVI_MYD13Q1_-_16-daily_2021.07.04.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MYD13/NDVI_MYD13Q1_-_16-daily_2021.07.20.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MYD13/NDVI_MYD13Q1_-_16-daily_2021.08.05.tif',
#     '/Users/hmcoerver/On My Mac/pyWAPOR/example_data/RAW/MODIS/MYD13/NDVI_MYD13Q1_-_16-daily_2021.06.02.tif']]
    
#     temp_folder = None
#     example_ds = None
#     lean_output = False 
#     diagnostics = None

#     main(cmeta, dbs, epochs_info, lean_output = False)


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MCD43

Description:
This module downloads MCD43 ALBEDO data from
https://e4ftl01.cr.usgs.gov/. Use the MCD43.ALBEDO function to
download and create daily ALBEDO images in Gtiff format.
The data is available between 2000-02-18 till present.

Examples:
from pyWAPOR.Collect import MCD43
MCD43.ALBEDO(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2003-12-20',
           latlim=[41, 45], lonlim=[-8, -5])
"""

from .ALBEDO import main as ALBEDO

__all__ = ['ALBEDO']

__version__ = '0.1'



# -*- coding: utf-8 -*-
import sys
from pywapor.collect.GEOS.DataAccess import DownloadData
import tqdm
import pandas as pd

def main(Dir, latlim, lonlim, Startdate, Enddate, Vars, Waitbar = True):
    """
    This function downloads GEOS daily data for a given variable, time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Vars -- ['t2m', 'v2m'] 
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Waitbar -- 1 (Default) Will print a waitbar
    """
    no_dates = len(pd.date_range(Startdate, Enddate, freq="D"))
    no_vars = len(Vars)

    if Waitbar:
        waitbar = tqdm.tqdm(total = no_vars * no_dates, delay = 30, position = 0)
    else:
        waitbar = False

    all_files = dict()
    for Var in Vars:

        if Waitbar:
            waitbar.set_description_str(Var)

        # Download data
        all_files = {**all_files, **DownloadData(Dir, Var, Startdate, Enddate, latlim, lonlim, "daily", Period = '', Waitbar = waitbar)}


    return all_files

if __name__ == '__main__':
    main(sys.argv)



outputs = {
"vc": False,                  # Vegetation Cover [-]
"lai": False,                 # Leaf Area Index [-]
"lai_eff": False,             # Effective Leaf Area Index [-]
"sf_soil": False,             # Soil Fraction [-]  
"lat": False,                 # Latitude [degrees]
"slope": False,               # Slope [rad]
"aspect": False,              # Aspect []
"ra_24_toa": False,           # Daily Solar Radiation At The Top Of Atmosphere [Wm-2]
"ws": False,                  # Sunset Hour Angle [Rad]
"diffusion_index": False,     # Diffusion Index 
"ra_24": False,              # Daily Solar Radiation For A Flat Surface [Wm-2]
"ra_24_toa_flat": False,      #
"stress_rad": False,          # Stress Factor For Radiation [-]
"p_air_24": False,            # Daily Air Pressure [mbar]
"vp_24": False,               # Daily Vapour Pressure [mbar]
"svp_24": False,              # Daily Slope Of Saturated Vapour Pressure Curve [mbar K-1]
"vpd_24": False,              # Daily Vapour Pressure Deficit [mbar]
"stress_vpd": False,          # Stress Factor For Vapour Pressure Deficit [-]
"stress_temp": False,         # Stress Factor For Air Temperature [-]   
"r_canopy_0": False,          # Atmospheric Canopy Resistance [sm-1]
"t_air_k_24": False,          # Daily Air Temperature [Kelvin]
"l_net": False,               # Daily Net Longwave Radiation [wm-2]
"int_mm": True,              # Interception [mm day-1]
"lh_24": False,               # Daily Latent Heat For Evaporation [j kg-1]
"int_wm2": False,            # Interception [Wm-2]
"rn_24": False,               # Net Radiation [Wm-2]      
"rn_24_canopy": False,        # Net Radiation For The Canopy [Wm-2]          
"t_air_k_i": False,           # Instantaneous Air Tempearture [K]
"vp_i": False,                # Instantaneous Vapour Pressure [mbar]
"ad_moist_i": False,          # Instantaneous Moist Air Density [kg m-3]
"ad_dry_i": False,            # Instantaneous Dry Air Density [kg m-3]
"ad_i": False,                # Instantaneous Air Density [kg m-3]
"u_b_i_bare": False,          # Instantaneous Wind Speed At Blending Height For Bare Soil [m s-1]
"lon": False,                 # longitude [degrees]
"ha": False,                  # Solar Hour Angel [rad]
# ied": False,                 # Inverse Earth Sun Distance [A U-1]
"h0": False,                  # Solar Elevation Angle [degrees]
"h0ref": False,               # Solar Elevation Angle Corrected For Refraction [degrees]
"m": False,                   # Relative Optical Airmass [-]
"rotm": False,                # Rayleigh Optical Thickness At Airmass m [-]
"Tl2": False,                # Airmass 2 Linke Athmospheric Turbidity Factor [-]
"B0c": False,                 # Beam Irradiance Normal To The Solar Beam [W m-2]
"Bhc": False,                 # Beam Irradiance At A Horizontal Surface [W m-2]
"Dhc": False,                 # Diffuse Irradiance At A Horizontal Surface [W/m2]
"ra_hor_clear_i": False,      # Total Clear-sky Irradiance On A Horizontal Surface
"emiss_atm_i": False,        # Instantaneous Athmospheric Emissivity [-]
"rn_bare": False,             # Net Radiation Bare Soil [W m-2]
"rn_full": False,             # Net Radiation Full Vegetation [W m-2]
"u_b_i_full": False,          # Instantaneous Wind Speed At Blending Height For Full Vegetation [m s-1]
"u_star_i_bare": False,       # Instantaneous Friction Velocity Bare Soil [m s-1]
"u_star_i_full": False,       # Instantaneous Friction Velocity Vegetation [m s-1]
"u_i_soil": False,            # Instantaneous Wind Speed Just Above Soil Surface [m s-1]
"ras": False,                # Aerodynamical Resistance [s m-1]
"raa": False,                 # Aerodynamical Resistance [s m-1]
"rac": False,                 # Aerodynamical Resistance Canopy [s m-1]        
"t_max_bare": False,          # Maximum Temperature At Bare Soil [K]
"t_max_full": False,          # Maximum Temperature At Full Dry Vegetation [K]
"t_min_bare": False,          # Minimum Temperature At Bare Soil [K]
"t_min_full": False,          # Minimum Temperature At Full Dry Vegetation [K]
"w_i": False,
"t_dew_i": False,             # Instantaneous Dew Point Temperature [C]
"t_wet_i": False,             # Instantaneous Wet Bulb Temperature [C]
"t_wet_k_i": False,           # Instantaneous Wet Bulb Temperature [K]
"lst_max": False,            # Maximum Temperature At Dry Conditions [K]
"lst_min": False,            # Minimum Temperature At Dry Conditions [K]
"se_root": True,             # Relative Root Zone Soil Moisture Based On LST [%] 
"stress_moist": False,        # Stress Factor For Root Zone Moisture [-]        
"r_canopy_0": False,          # Atmospheric Canopy Resistance [s m-1]
"r_canopy": False,            # Canopy Resistance [s m-1]
"z_obst": False,              # Obstacle Height [m]
"z_oro": False,               # Orographic Roughness [m]
"z0m": False,                 # Roughness Length [m]
"ra_canopy_init": False,      # Canopy Resistance Without Stability Corrections [s m-1]
"u_b_24": False,              # Daily Wind Speed At Blending Height [m s-1]
"disp": False,                # Displacement Height [m]
"u_star_24_init": False,      # Initial Estimate Of The Daily Friction Velocity [m s-1] 
"ad_dry_24": False,           # Daily Dry Air Density [kg m-3]
"ad_moist_24": False,         # Daily Moist Air Desity [kg m-3]
"ad_24": False,               # Daily Air Density [kg m-3]
"psy_24": False,              # Daily Psychrometric Constant [mbar K-1]
"ssvp_24": False,             # Daily Slope Of Saturated Vapour Pressure Curve [mbar K-1]
"t_24_init": False,           # Initial Estimate Radiation Equivalent Daily Transpiration [W m-2]
"h_canopy_24_init": False,    # Initial Estimate Of Sensible Heat Flux
"t_24": False,                # Daily Transpiration Energy Equivalent [W m-2]
"t_24_mm": True,             # Daily Transpiration [mm day-1]
"tpot_24": False,     
"tpot_24_mm": False,
"sf_soil": False,             # Soil Fraction [-]
"rn_24_soil": False,          # Net Radiation For The Soil [W m-2]
"r_soil": False,              # Soil Resistance [s m-1]
"ra_soil_init": False,        # Aerodynamic Resistance Withou Stability Corrections [s m-1]
"u_b_24": False,              # Daily Wind Speed At Blending Height [m]
"u_star_24_soil_init": False, # Initial Estimate Of The Daily Friction Velocity For Soil [m s-1]
"g0_bs": False,               # Bare Soil Heat Flux [W m-2]
"g0_24": False,               # Daily Soil Heat Flux G [W m-2]
"e_24_init": False,           # Initial Estimate Radiation Equivalent Daily Evaporation [W m-2]
"h_soil_24_init": False,      # Initial Estimate Of The Sensible Heat Flux For Soil [W m-2]
"e_24": False,                # Daily Evaporation Energy Equivalent [W m-2]
"e_24_mm": True,             # Daily Evaporation [mm day-1]
"et_24_mm": True,            # Daily Evapotranspiration [mm day-1]
"rn_24_grass": False,         # Daily Net Radiation For Reference Grass [W m-2]
"et_ref_24": False,           # Reference Evapotranspiration envergy equivalent [W m-2]
"et_ref_24_mm": True,        # Reference Evapotranspiration [mm day-1]
"biomass_prod": True,        # Biomass Production [kg/ha]
"L_bare": False,
"L_full": False,
"trans_24": False,
}


import requests
import pywapor
import os
import time
import numpy as np
from datetime import datetime as dat

def check_source_selection(source_selection, startdate, enddate):

    if isinstance(startdate, str):
        startdate = dat.strptime(startdate, "%Y-%m-%d").date()
    if isinstance(enddate, str):
        enddate = dat.strptime(enddate, "%Y-%m-%d").date()

    valid_sources, valid_dates = pywapor.general.variables.get_source_validations()

    temporal_sources = ["METEO", "NDVI", "ALBEDO", "LST", 
                    "PRECIPITATION", "TRANS"]

    check_keys = np.all([key in valid_sources.keys() for key in source_selection.keys()])
    assert check_keys, "invalid key in source_selection"

    assert len(source_selection["DEM"]) == 1, "only one DEM source can be selected"
    assert len(source_selection["METEO"]) == 1, "only one METEO source can be selected"
    assert len(source_selection["PRECIPITATION"]) == 1, "only one PRECIPITATION source can be selected"
    assert len(source_selection["LULC"]) == 1, "only one LULC source can be selected"
    assert len(source_selection["TRANS"]) == 1, "only one TRANS source can be selected"
    assert len(source_selection["NDVI"]) >= 1, "only one TRANS source can be selected"
    assert len(source_selection["ALBEDO"]) >= 1, "only one TRANS source can be selected"
    assert len(source_selection["LST"]) >= 1, "only one TRANS source can be selected"

    results = dict()
    all_results = list()

    for var, sources in source_selection.items():

        check1 = [source in valid_sources[var] for source in sources]
        if var in temporal_sources:
            check2 = [startdate >= valid_dates[source][0] for source in sources]
            check3 = [enddate <= valid_dates[source][1] for source in sources]
        else:
            check2 = [True]
            check3 = [True]

        results[var] = {source: {"valid_source:": check1[i],
                                 "valid_startdate:": check2[i],
                                 "valid_enddate:": check3[i],
                                } for i, source in enumerate(sources)}

        all_results.append(np.all([check1, check2, check3]))
    
    succes = np.all(all_results)

    return results, succes

def nasa_account(user_pw = None):

    n_max = 3
    succes = False
    n = 1

    while not succes and n <= n_max:

        folder = os.path.dirname(os.path.realpath(pywapor.__path__[0]))
        test_url = r"https://goldsmr4.gesdisc.eosdis.nasa.gov/data/MERRA2/M2T3NXGLC.5.12.4/1987/08/MERRA2_100.tavg3_2d_glc_Nx.19870801.nc4"
        test_file = os.path.join(folder, "nasa_test.nc4")

        if os.path.isfile(test_file):
            os.remove(test_file)

        if not isinstance(user_pw, type(None)):
            username, password = user_pw
        else:
            username, password = pywapor.collect.get_pw_un.get('NASA')

        x = requests.get(test_url, allow_redirects = False)
        y = requests.get(x.headers['location'], auth = (username, password))

        if x.ok and y.ok:

            with open(test_file, 'w+b') as z:
                z.write(y.content)
        
            if os.path.isfile(test_file):
                statinfo = os.stat(test_file)
                succes = statinfo.st_size == 3963517
                os.remove(test_file)
                if not succes:
                    error = "please add 'NASA GESDISC DATA ARCHIVE' to 'Approved Applications'."
            else:
                succes = False

        else:
            error = "wrong username/password."
            succes = False
            if os.path.isfile(test_file):
                os.remove(test_file)

        if not succes:
            print(f"Try {n}/{n_max} failed: {error}")
            time.sleep(3)
            
        n += 1

    return succes

# nasa_account()